
var game = new Phaser.Game(1024, 768, Phaser.AUTO, '', { preload: preload, create: create, update: update });

var btnHeart;
var currentPoemData;
var currentSlotPositions;
var correctSoundOrder;
var puzzleBubble;
var bardomatic;
var robotSprite;
var winVideo;
var wordBin = {x:50, y: 660, width:700, height: 60};
var laundryTween;
var laundrySprite;
var eyeAnimArray;

function preload() {

  game.load.image('sky', 'assets/images/sky.png');
  game.load.image('cloud-1', 'assets/images/cloud-1.png');
  game.load.image('cloud-2', 'assets/images/cloud-2.png');
  game.load.image('laundry', 'assets/images/robot-2.png');
  game.load.image('Tower-ground', 'assets/images/Tower-ground.png');

  game.load.image('bubble-with-map', 'assets/images/bubble-with-map.png');

  game.load.spritesheet('robot', 'assets/images/robot.png', 441, 353, 12);
  game.load.spritesheet('bardomatic_noTxt', 'assets/images/bard-fly-in.png', 540, 360, 2);
  game.load.spritesheet('bardomatic_scroll', 'assets/images/bard-words.png', 321, 75, 240);

  game.load.image('btnHeart', 'assets/images/btnHeart.png');

  // Load Word Assets
  game.load.image('w_our', 'assets/images/words/our.png');
  game.load.image('w_puzzle', 'assets/images/words/puzzle.png');
  game.load.image('w_otherwise', 'assets/images/words/otherwise.png');
  game.load.image('w_ah', 'assets/images/words/ah.png');
  game.load.image('w_complete', 'assets/images/words/complete.png');
  game.load.image('w_feel', 'assets/images/words/feel.png');
  game.load.image('w_fit', 'assets/images/words/fit.png');
  game.load.image('w_grooves', 'assets/images/words/grooves.png');
  game.load.image('w_grooves_ah', 'assets/images/words/grooves_ah.png');
  game.load.image('w_to', 'assets/images/words/to.png');
  game.load.image('w_pieces', 'assets/images/words/pieces.png');
  game.load.image('w_unseen', 'assets/images/words/unseen.png');

  // Load Word Sounds
  game.load.audio('snd_our', 'assets/sounds/our.mp3');
  game.load.audio('snd_puzzle', 'assets/sounds/puzzle.mp3');
  game.load.audio('snd_otherwise', 'assets/sounds/otherwise.mp3');
  game.load.audio('snd_ah', 'assets/sounds/ah.mp3');
  game.load.audio('snd_complete', 'assets/sounds/complete.mp3');
  game.load.audio('snd_feel', 'assets/sounds/feel.mp3');
  game.load.audio('snd_fit', 'assets/sounds/fit.mp3');
  game.load.audio('snd_grooves', 'assets/sounds/grooves.mp3');
  game.load.audio('snd_to', 'assets/sounds/to.mp3');
  game.load.audio('snd_pieces', 'assets/sounds/pieces.mp3');
  game.load.audio('snd_unseen', 'assets/sounds/unseen.mp3');
  game.load.audio('snd_the whole business', 'assets/sounds/the whole business.mp3');

  // Load video
  game.load.video('winVideo', 'assets/video/win.mp4');

}

function create() {

  game.add.sprite(0, 0, 'sky');
  game.tweens.create(game.add.sprite(1074, 100, 'cloud-1')).to({x: -332}, 100000, null, true, 0, -1);
  game.tweens.create(game.add.sprite(1074, 50, 'cloud-2')).to({x: 355}, 70000, null, false, 0, -1).to({x: -363}, 70000).start(1);
  laundrySprite = game.add.sprite(60, 26, 'laundry');
  restartLaundryTween();
  game.add.sprite(0, 0, 'Tower-ground');
  robotSprite = game.add.sprite(580, 292, 'robot');
  puzzleBubble = game.add.sprite(180, 48, 'bubble-with-map');

  btnHeart = game.add.button(722, 392, 'btnHeart', onConnectBtnClick, this, 2, 1, 0);

  // Go first level
  buildPoemForLevel(1);

  // Set up robot animations
  robotSprite.animations.add('heart', [0,1], 2, true, true);
  robotSprite.animations.add('eyes-ahead', [2], 6, true, true);
  robotSprite.animations.add('eyes-left', [3,4], 6, true, true);
  robotSprite.animations.add('eyes-down', [5], 6, true, true);
  robotSprite.animations.add('eyes-left-up', [6], 6, true, true);
  robotSprite.animations.add('eyes-front', [7], 6, true, true);
  robotSprite.animations.add('eyes-rock', [8,9,10,11], 6, true, true);
  eyeAnimArray = ['eyes-left', 'eyes-down', 'eyes-left-up', 'eyes-front', 'eyes-rock'];
  robotSprite.animations.play('heart');

  bardomatic = game.add.sprite(250, 50, 'bardomatic_noTxt', 5);
  anim = bardomatic.animations.add('fly-in');
  anim.play(2, true);

  bardomaticScroll = game.add.sprite(360, 230, 'bardomatic_scroll', 5);

  startIntroSequence();

  winVideo = game.add.video('winVideo');

}

function update() {

}

function soundSortByPosition(a,b) {
  if (a.y == b.y) return a.x - b.x;
  return a.y - b.y;
}

function getSoundOrder(slotPositions, correct) {
  var sounds = [];

  for (var i = 0; i < slotPositions.length; i++) {
    var x;
    var y;
    if (correct) {
      x = slotPositions[i].correct_x;
      y = slotPositions[i].correct_y;
    }
    else
    {
      x = slotPositions[i].sprite.x;
      y = slotPositions[i].sprite.y;
    }
    if (slotPositions[i].hasOwnProperty('sounds'))
    {
      for (var j = 0; j < slotPositions[i].sounds.length; j++) {
        var sound = slotPositions[i].sounds[j];
        sounds.push({soundid: 'snd_' + sound.sound, x: x + sound.x * 80, y: y + sound.y * 80});
      }
    }
    else
    {
      sounds.push({soundid: 'snd_' + slotPositions[i].assetId, x: x, y: y})
    }
  };

  sounds.sort(soundSortByPosition);

  console.log(sounds);

  var result = [];
  for (var i=0; i<sounds.length; i++)
  {
    result.push(sounds[i].soundid);
  }
  return result;
}

// Build a poem
function buildPoemForLevel(levelNum) {

  currentPoemData = poems['poem' + levelNum];
  var author = currentPoemData.author;

  var rows = currentPoemData.puzzle.slots;

  var rowHeight = 80;
  var colWidth = 80;

  game.add.audio('snd_' + currentPoemData.puzzle.successSnd);

  var slotPositions = [];
  var colPosition = 0;

  for (var i = 0; i < rows.length; i++) {

    var row = rows[i];
    colPosition = 0;

    for (var k in row) {
      if (row.hasOwnProperty(k)) {

        var word = k;
        var syllCount = row[k];

        if (word === 'EMPTY') {
          // console.log('Empty slot found');
        } else {

          var assetId = word;
          var newSlot = {id: word, assetId:assetId, syllCount:syllCount, x:colPosition * colWidth, y:i * rowHeight, w:colWidth * syllCount, h:rowHeight};

          if (currentPoemData.puzzle.attached_sounds.hasOwnProperty(word)) {
            newSlot.sounds = currentPoemData.puzzle.attached_sounds[word];
          }

          newSlot.correct_x = newSlot.x + currentPoemData.puzzle.x;
          newSlot.correct_y = newSlot.y + currentPoemData.puzzle.y;

          addWordPiece(newSlot);
          addWordSound(newSlot);

          slotPositions.push(newSlot);

        }

        colPosition += syllCount;

      } else {
        console.log('warning- key not found');
      }

    }

  };

  currentSlotPositions = slotPositions;

  correctSoundOrder = getSoundOrder(currentSlotPositions, true);

}

function onWordPressed(sprite, pointer) {
  sprite.bringToTop();
}

function addWordPiece(slot) {

  var rX = wordBin.x + Math.random() * wordBin.width;
  var rY = wordBin.y + Math.random() * wordBin.height;

  var newWord = game.add.sprite(rX, rY, 'w_' + slot.assetId);
  slot.x = rX;
  slot.y = rY - newWord.height;
  newWord.inputEnabled = true;
  newWord.input.enableDrag();
  newWord.input.enableSnap(80, 80, false, true, 0, 0);
  newWord.events.onInputDown.add(onWordPressed);

  slot.sprite = newWord;

}

function addWordSound(slot) {

  if (slot.hasOwnProperty('sounds')) {
    for (var sound in slot.sounds) {
      game.add.audio('snd_' + sound);
    }
  }
  else
    game.add.audio('snd_' + slot.assetId);
}

function snapWordOnDrop(item) {

  console.log('dropped:', item);

}

var current_sound;
var remaining_sounds;
function play_sounds(sounds) {
  if (sounds.length != 0) {
    current_sound = game.sound.play(sounds[0]);
    remaining_sounds = sounds.slice(1);
    current_sound.onStop.addOnce(on_stop);
    jiggleWord(sounds[0]);
  } else {
    console.log('All sounds have played');
    onPoemCompletelyRead();
  }
}

function on_stop() {
  play_sounds(remaining_sounds);
}

function restartLaundryTween() {

  laundryTween = game.tweens.create(laundrySprite).to({x: 53, y:32}, 1300, Phaser.Easing.Quadratic.InOut, true, Math.ceil(Math.random()*10000), 0, true);
  laundryTween.onComplete.add(restartLaundryTween, this);

}

function randomEyeMovement() {

  robotSprite.animations.play('eyes-ahead');

  setTimeout(function () {

    var animId = eyeAnimArray[Math.floor(Math.random()*eyeAnimArray.length)];
    robotSprite.animations.play(animId);

    setTimeout(randomEyeMovement, 500 + Math.ceil(Math.random()*1700))

  }, 5000 + Math.ceil(Math.random()*5000));

}

function findSlotForSound(snd) {
  var slot;

  for (var i = 0; i < currentSlotPositions.length; i++) {
    if (currentSlotPositions[i].hasOwnProperty('sounds')) {
      for (var j=0; j < currentSlotPositions[i].sounds.length; j++) {
        if ('snd_' + currentSlotPositions[i].sounds[j].sound == snd) {
          slot = currentSlotPositions[i];
        }
      }
    }
    else if ('snd_' + currentSlotPositions[i].assetId === snd) {
      slot = currentSlotPositions[i];
      break;
    }
  };

  return slot;
}

function jiggleWord(snd) {

  var wordSprite = findSlotForSound(snd).sprite;

  game.add.tween(wordSprite).from({y: wordSprite.y - 10}, 700, Phaser.Easing.Elastic.Out, true, 200, 0, false);

}

function wordSortByPosition(a,b) {
  if (a.sprite.y == b.sprite.y) return a.sprite.x - b.sprite.x;
  return a.sprite.y - b.sprite.y;
}

function onConnectBtnClick() {

  currentSlotPositions.sort(wordSortByPosition);

  var sndArray = getSoundOrder(currentSlotPositions, false);

  if (getMismatched(sndArray, correctSoundOrder).length === 0){
    // Order is correct! play completed sound
    console.log('success!');
    game.sound.play('snd_' + currentPoemData.puzzle.successSnd);

    for (var j = 0; j < currentSlotPositions.length; j++) {
      var wordSprite = currentSlotPositions[j].sprite;
      game.add.tween(wordSprite).from({y: wordSprite.y - 7}, 350, Phaser.Easing.Bounce.Out, true, (j*15), -1, true);
    };

    startEndingSequence();

  } else {
    // Play individual word sounds
    play_sounds(sndArray);

  }

  btnHeart.alpha = 0.35;
  btnHeart.inputEnabled = false;

}

function onPoemCompletelyRead() {

  var incorrectWords = [];

  for (var i=0; i<currentSlotPositions.length; i++) {
     if (Math.abs(currentSlotPositions[i].sprite.x - currentSlotPositions[i].correct_x) > 2 ||
         Math.abs(currentSlotPositions[i].sprite.y - currentSlotPositions[i].correct_y) > 2)
       incorrectWords.push(currentSlotPositions[i]);
  }

  dropIncorrectWords(incorrectWords);

  btnHeart.alpha = 1.0;
  btnHeart.inputEnabled = true;


}

function startIntroSequence() {

  dropPoemWords();
  puzzleBubble.visible = false;
  bardomatic.visible = false;
  btnHeart.visible = false;
  bardomaticScroll.visible = false;

  setTimeout( function () {

    // drop in bardo
    bardomatic.visible = true;
    game.add.tween(bardomatic).from({y: -400}, 1000, Phaser.Easing.Circular.Out, true, 0, 0, false);

    setTimeout(function () {
      bardomaticScroll.visible = true;
      var scrollAnim = bardomaticScroll.animations.add('scroll-in');
      scrollAnim.play(12, false);
    }, 1300);

    setTimeout( function () {

      game.add.tween(bardomatic).to({y: -400}, 800, Phaser.Easing.Circular.In, true, 0, 0, false);

      setTimeout(function () {

        togglePuzzleDisplay(true);
        randomEyeMovement();

      }, 3000);

    }, 22000);

  }, 500);

}

function startEndingSequence () {

  setTimeout(function () {

    togglePuzzleDisplay(false);

    setTimeout(function () {
      robotSprite.animations.play('heart');
    }, 1000);

    setTimeout(function () {
      winVideo.play(false);
      winVideo.addToWorld(0,0);
    }, 4700);

  }, 15000);


}


function dropPoemWords() {

  for (var i = 0; i < currentSlotPositions.length; i++) {
    var spr = currentSlotPositions[i].sprite;

    spr.alpha = 0.0;
    spr.x = 430;
    spr.y = 380 - spr.height;

    // Add a simple bounce tween to each sprite's position.
    var delay = 7000 + 50 * i;
    game.add.tween(spr).to({x:currentSlotPositions[i].x, alpha:1.0}, 900, Phaser.Easing.Quadratic.Out, true, delay, 0, false);
    game.add.tween(spr).to({y: currentSlotPositions[i].y}, 900, Phaser.Easing.Bounce.Out, true, delay, 0, false);

  };

}

function dropIncorrectWords(incorrectWordSlots) {

  for (var i = 0; i < incorrectWordSlots.length; i++) {
    var spr = incorrectWordSlots[i].sprite;

    // Add a simple bounce tween to each sprite's position.
    var rY = wordBin.y + Math.random() * wordBin.height - spr.height;
    var delay = 100 + 150 * i;
    game.add.tween(spr).to({y: rY}, 900, Phaser.Easing.Bounce.Out, true, delay, 0, false);

  };

}

function togglePuzzleDisplay(doShow) {

  if (doShow === true){
    puzzleBubble.visible = true;
    btnHeart.visible = true;
    game.add.tween(btnHeart).from({width:0, height:0, x: btnHeart.x + btnHeart.width/2, y: btnHeart.y  + btnHeart.height/2}, 1100, Phaser.Easing.Elastic.Out, true, 900, 0, false);
    game.add.tween(puzzleBubble).from({width:0, height:0, x: puzzleBubble.x + puzzleBubble.width, y: puzzleBubble.y  + puzzleBubble.height - 25}, 800, Phaser.Easing.Quadratic.Out, true, 0, 0, false);

  } else {
    game.add.tween(btnHeart).to({width:0, height:0, x: btnHeart.x + btnHeart.width/2, y: btnHeart.y  + btnHeart.height/2}, 1100, Phaser.Easing.Elastic.Out, true, 900, 0, false);
    game.add.tween(puzzleBubble).to({width:0, height:0, x: puzzleBubble.x + puzzleBubble.width, y: puzzleBubble.y  + puzzleBubble.height - 25}, 800, Phaser.Easing.Quadratic.Out, true, 0, 0, false);

    for (var i=0; i<currentSlotPositions.length; i++) {
      var slotSprite = currentSlotPositions[i].sprite;
      game.add.tween(slotSprite).to({width:0, height:0, x: laundrySprite.x + 60, y: laundrySprite.y + 20}, 2000, Phaser.Easing.Quadratic.In, true, 150 + (i*50), 0, false);
    }
  }

}

// arr 1 is test array. arr2 is 'correct' order.
function getMismatched(arr1, arr2) {
  var mismatchedArray = [];

  for (var i = arr1.length; i--;) {
    if (arr1[i] !== arr2[i]) {
      mismatchedArray.push(arr1[i]);
    }
  }

  return mismatchedArray;
}

